/*********************************************
* vim: set sw=8 ts=8 si et :
* Author: Guido Socher, Copyright: GPL 
* This program is to test the led connected to LEDOUT (see below)
* 
* Clock frequency     : Internal clock 1 Mhz (factory default)
*********************************************/
#include <avr/io.h>
#define F_CPU 1000000UL  // 1 MHz
#include <util/delay.h>

#define BUZZOUT PORTB1
#define BUZZPORT PORTB
#define BUZZDDR DDRB
#define BUZZDDRPIN DDB1
#define LEDOUT PORTB2
#define LEDPORT PORTB
#define LEDDDR DDRB
#define LEDDDRPIN DDB1

unsigned int ledintensity=0; // 16 bit var and 10bit pwm

// _delay_ms uses a floating point datatype if you call
// that function in many places in your code then it becomes
// very fat. An integer is enough for us:
//
// delay x milliseconds:
void delay_ms(unsigned int xms)
{
        while(xms){
                _delay_ms(0.96);
                xms--;
        }
}

uint8_t getdoorstate(void)
{
  // PD2= GND: blink, PD2= Vcc fading light
  if (bit_is_clear(PIND,PIND2)){
    return(0);
  }
  return(1);
}


uint8_t buzzflip( uint8_t old_state )
{
  if( old_state ){
    BUZZPORT &= ~(1<<BUZZOUT);
    return 0;
  }
  BUZZPORT|= (1<<BUZZOUT);
  return 1;
}

uint8_t flashflip( uint8_t old_state )
{
  if( old_state ){
    LEDPORT &= ~(1<<LEDOUT);
    return 0;
  }
  LEDPORT|= (1<<LEDOUT);
  return 1;
}

void buzzset( uint8_t set_state )
{
  if( set_state ){
    BUZZPORT|= (1<<BUZZOUT);
  }else{
    BUZZPORT &= ~(1<<BUZZOUT);
  }
}

void flashset( uint8_t set_state )
{
  if( set_state ){
    LEDPORT|= (1<<LEDOUT);
  }else{
    LEDPORT &= ~(1<<LEDOUT);
  }
}

void resettimer(void){
  TCNT1 = 0; // Reset timer value 
}

int main(void)
{
  uint8_t door_open=0;
  uint8_t last_door_state = 0;
  int buzz_for = 0;
  int buzz_gap = 0;
  uint8_t buzz_state = 0;
  uint32_t last_buzz_at = 0;
  int flash_for = 0;
  int flash_gap = 0;
  uint8_t flash_state = 0;
  uint32_t last_flash_at = 0;

  uint32_t alarm_ms = 0;
  // Set up timer at Fcpu/64
  TCCR1B |= (1 << CS10);

  /* enable pins as output */
  BUZZDDR|= (1<<BUZZDDRPIN);
  LEDDDR|= (1<<LEDDDRPIN);
  // PD2 to check the mode:
  DDRD&= ~(1<<PIND2);
  PORTD|=1<<PIND2; // internal pullup resistor on

  while( 1 )
  {
    door_open=getdoorstate();
    // If the door state has changed, reset the timer
    if( door_open != last_door_state ){
      resettimer();
      last_door_state=door_open;
      flashset( 1 );
      delay_ms( 100 );
      flashset( 0 );
      
      if( door_open ){
        alarm_ms = 0;
        last_flash_at = 0;
        last_buzz_at = 0;
        buzz_gap = 0;
        buzz_for = 0;
        flash_gap = 1950;
        flash_for = 50;
      }
      else
      {
        flash_gap = 0;
        buzz_gap = 0;
      }
    }

    // set the alarm level
    if( door_open ){
      // Count the milliseconds - one millisecond per 1000 clock cycles
      if( TCNT1 >= 1000 && alarm_ms < 2147483647 ){
        alarm_ms++;
        resettimer();
      }

      // Set alarm intensity depending on time which alarm has been going
      switch( alarm_ms )
      {
        case 10000: // 10 seconds
          flash_gap = 500;
          flash_for = 500;
          break;
        case 15000: // 12 seconds
          flash_gap = 50;
          flash_for = 50;
          break;
        case 20000: // 20 seconds
          buzz_gap = 2900;
          buzz_for = 200;
          break;
        case 60000: // 60 seconds
          buzz_gap = 300;
          buzz_for = 200;
          break;
      }

      // turn buzzer on/off
      if( buzz_gap > 0 &&
        ( ( buzz_state && ( alarm_ms - last_buzz_at > buzz_for ) ) ||
          ( ! buzz_state && ( alarm_ms - last_buzz_at > buzz_gap ) ) ) ){
        last_buzz_at = alarm_ms;
        buzz_state = buzzflip( buzz_state );
      }

      // turn buzzer on/off
      if( flash_gap > 0 &&
        ( ( flash_state && alarm_ms - last_flash_at > flash_for ) ||
          ( ! flash_state && alarm_ms - last_flash_at > flash_gap ) ) ){
        last_flash_at = alarm_ms;
        flash_state = flashflip( flash_state );
      }
    }else{
      // make sure flash and buzzer are off
      if( flash_state ){
        flash_state = 0;
        flashset( 0 );
      }
      if( buzz_state ){
        buzz_state = 0;
        buzzset( 0 );
      }
    }
  }
  return(0);
}

